#include <Graphics\UI.h>
#include <Log\Logger.h>

#include <string>
#include <vector>

Microsoft::WRL::ComPtr<ID2D1DeviceContext2> pD2DDeviceCont{ nullptr };
Microsoft::WRL::ComPtr<IDWriteFactory> pDWriteFactor{nullptr};

POINT p{};

void SetGUIContext(Microsoft::WRL::ComPtr<ID2D1DeviceContext2> pD2DCont, Microsoft::WRL::ComPtr<IDWriteFactory> pDWFactory) {
	pD2DDeviceCont = pD2DCont;
	pDWriteFactor = pDWFactory;
}

void SetGUIMouse(POINT pt) {
	p = pt;
}

void DeleteGUIContext() {
	Log_Write("** Releasing of GUI **");
	p.x = p.y = 0;
	pD2DDeviceCont.Reset();
	Log_Write("\tDevice Context Released");
	pDWriteFactor.Reset();
	Log_Write("\tWrite Factor Released");

	Log_Write("** GUI Released **");
}

std::vector<Control> ctrl{};

bool CreateUXWindow(unsigned short type, std::wstring name, DWORD style, int X, int Y, int W, int H, UINT32 f_id, Window parent) {
	Control wnd{};

	// Window Default parameters

	ErrorHR(pD2DDeviceCont->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::WhiteSmoke), &wnd.pTextBrush));
	ErrorHR(pD2DDeviceCont->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::White), &wnd.BorderBrush));
	ErrorHR(pD2DDeviceCont->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::DarkGray), &wnd.BorderSelectBrush));
	ErrorHR(pDWriteFactor->CreateTextFormat(L"Verdana", NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL, DWRITE_FONT_STRETCH_NORMAL, 24, L"pl-PL", &wnd.pTextFormat));
	ErrorHR(wnd.pTextFormat->SetTextAlignment(DWRITE_TEXT_ALIGNMENT_CENTER));
	ErrorHR(wnd.pTextFormat->SetParagraphAlignment(DWRITE_PARAGRAPH_ALIGNMENT_CENTER));

	// Current Control params

	wnd.SetName(name);
	wnd.SetBorder(true);
	wnd.SetID(f_id);
	wnd.SetType(type);
	wnd.SetWindowRect(X, Y, W, H);

	wnd.Wnd = D2D1::RectF(static_cast<FLOAT>(X), static_cast<FLOAT>(Y), static_cast<FLOAT>(X + W), static_cast<FLOAT>(Y + H));

	ctrl.push_back(wnd);

	return true;
}

bool CreateUXWindow(std::wstring cname, std::wstring name, DWORD style, int X, int Y, int W, int H, UINT32 f_id) {

	return true;
}

void DestroyAllWindows() {
	ctrl.clear();
	Log_Write("** All Controls are Released **");
}

void RenderStatic(Control &current, Window Parent) {
	if (current.HasBorder()) {
		pD2DDeviceCont->DrawRectangle(&current.Wnd, current.BorderBrush.Get(), 2, NULL);
	}

	pD2DDeviceCont->DrawTextW(current.GetName().c_str(), static_cast<UINT32>(wcslen(current.GetName().c_str())), current.pTextFormat.Get(), &current.Wnd, current.pTextBrush.Get());
}

void UpdateButton(Control &current, Window Parent) {
	if ((p.x >= current.Wnd.left) && (p.x <= current.Wnd.right) && (p.y <= current.Wnd.bottom) && (p.y >= current.Wnd.top)) {
		current.InRange(true);
	}
	else current.InRange(false);
}

void RenderButton(Control &current, Window Parent) {	
	if (current.IsInRange())
		pD2DDeviceCont->DrawRectangle(&current.Wnd, current.BorderSelectBrush.Get(), 2, NULL);
	else
		pD2DDeviceCont->DrawRectangle(&current.Wnd, current.BorderBrush.Get(), 2, NULL);

	pD2DDeviceCont->DrawTextW(current.GetName().c_str(), static_cast<UINT32>(wcslen(current.GetName().c_str())), current.pTextFormat.Get(), &current.Wnd, current.pTextBrush.Get());
}
void UpdateUI() {
	for (auto &e : ctrl) {
		if (e.GetType() == BUTTON) {
			UpdateButton(e, 0);
		}
	}
}

void RenderUI() {
	for (auto &e : ctrl) {
		if (e.GetType() == STATIC) {
			RenderStatic(e, 0);
		} else
		if (e.GetType() == BUTTON) {
			RenderButton(e, 0);
		}
	}
}

// New

void SetUXBorder (UINT32 wnd, bool border) {
	for (auto &e : ctrl) {
		if (e.GetId() == wnd) {
			e.SetBorder(border);
			return;
		}
	}
}

// About Brushes
// https://msdn.microsoft.com/en-us/library/windows/desktop/dd756651(v=vs.85).aspx

void SetUXBorderColor(UINT32 wnd, D2D1::ColorF clr) {
	for (auto &e : ctrl) {
		if (e.GetId() == wnd) {
			ErrorHR(pD2DDeviceCont->CreateSolidColorBrush(D2D1::ColorF(clr), &e.BorderBrush));

			return;
		}
	}
}

void SetUXName(UINT32 wnd, std::wstring name) {
	for (auto &e : ctrl) {
		if (e.GetId() == wnd) {
			e.SetName(name);
			return;
		}
	}
}

void SetUXFont(UINT32 wnd, std::wstring font, int size, D2D1::ColorF clr) {
	for (auto &e : ctrl) {
		if (e.GetId() == wnd) {
			ErrorHR(pDWriteFactor->CreateTextFormat(font.c_str(), NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL, DWRITE_FONT_STRETCH_NORMAL, size, L"pl-PL", &e.pTextFormat));
			ErrorHR(pD2DDeviceCont->CreateSolidColorBrush(D2D1::ColorF(clr), &e.pTextBrush));

			return;
		}
	}
}