#pragma once

#ifndef __D2D_UI__
#define __D2D_UI__

#include <Windows.h>
#include <wrl.h>

#include <d2d1.h>
#include <d2d1_3.h>
#include <dwrite.h>

#include <iostream>

const unsigned short STATIC{ 1000 };
const unsigned short BUTTON{ 1001 };

struct MyRect {
	int x{}, y{}, w{}, h{};
};

struct Window {
	Window() {}
	Window(int) {}
	~Window() {}
};

struct Control {
	Control() {}
	~Control() {}

private:
	bool Border{ false };
	bool visible{ false };

	std::wstring Name{ L"" };

	MyRect rc{};
	bool inrange{ false };

	unsigned int id{};
	unsigned short type{};
public:
	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> BorderBrush{ nullptr };
	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> BorderSelectBrush{ nullptr };
	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> pTextBrush{ nullptr };
	Microsoft::WRL::ComPtr<IDWriteTextFormat> pTextFormat{ nullptr };

	D2D1_RECT_F Wnd{};

public:
	void SetBorder(bool b) { Border = b; }
	void SetName(std::wstring str) { Name = str; }
	void SetID(unsigned int ID) { id = ID; }
	void InRange(bool b) { inrange = b; }
	void SetWindowRect(int X, int Y, int W, int H) {
		rc.x = X;
		rc.y = Y;
		rc.w = W;
		rc.h = H;
	}
	void SetType(unsigned short t) { type = t; }

	bool HasBorder() { return Border; }
	unsigned int GetId() { return id; }
	bool IsInRange() { return inrange; }
	std::wstring GetName() { return Name; }
	unsigned short GetType() { return type; }
	MyRect GetWindowRect() { return rc; }

	bool operator== (const Control &hWnd) { return (id == hWnd.id); }
	bool operator!= (const Control &hWnd) { return (id != hWnd.id); }
};

bool CreateUXWindow(unsigned short type, std::wstring name, DWORD style, int X, int Y, int W, int H, UINT32 f_id, Window parent);
bool CreateUXWindow(std::wstring cname, std::wstring name, DWORD style, int X, int Y, int W, int H, UINT32 f_id);

void SetGUIContext(Microsoft::WRL::ComPtr<ID2D1DeviceContext2> pD2DCont, Microsoft::WRL::ComPtr<IDWriteFactory> pDWFactory);
void SetGUIMouse(POINT);
void DeleteGUIContext();

void SetUXBorder(UINT32 wnd, bool border);
void SetUXBorderColor(UINT32 wnd, D2D1::ColorF clr);

void SetUXName(UINT32 wnd, std::wstring);

void SetUXFont(UINT32 wnd, std::wstring font, int size, D2D1::ColorF clr);

void DestroyAllWindows();

void UpdateUI();
void RenderUI();

#endif __D2D_UI__