#include <WinMain.h>

#include <Log\Logger.h>

#include <iostream>
#include <string>
#include <tchar.h>

#include <Dwmapi.h>

#pragma comment (lib, "Dwmapi.lib")

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_CREATE:

		return 0;

	case WM_QUIT:
	case WM_DESTROY:
	case WM_CLOSE:
		PostQuitMessage(0);

		return 0;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

WMainProg::WMainProg() {
}

WMainProg::~WMainProg() {
	Log_Write("** End of Program **");
	Log_Close();
	DestroyWindow(MainWnd);
	hInst = nullptr;
}

void WMainProg::SetWindowName(std::wstring name) {
	windowstitle = name;
#ifdef DEBUG

	windowstitle += L"(DBG)";

#endif // DEBUG
	SetWindowText(MainWnd, windowstitle.c_str());
}

bool WMainProg::Create(WMainClass wmc) {
	WNDCLASSEX WndClassEx{};

	SetWindowName(L"DirectX12");

	std::wstring classstitle{ windowstitle };

	hInst = GetModuleHandle(nullptr);

	WndClassEx.cbSize = sizeof(WNDCLASSEX);
	WndClassEx.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
	WndClassEx.lpfnWndProc = static_cast<WNDPROC> (MainWndProc);
	WndClassEx.hInstance = hInst;
	WndClassEx.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	WndClassEx.lpszClassName = classstitle.c_str();

	if(!RegisterClassEx(&WndClassEx)) {
		Logger("\tRegisterClassEx failed");
		return false;
	}

	DEVMODE displayMode{};
	displayMode.dmSize = sizeof(DEVMODE);

	if (!EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &displayMode)) {
		Logger("\tEnumDisplaySettings failed");

		return false;
	}

	int X{}, Y{};

	if (wmc.Width != displayMode.dmPelsWidth)
		X = (GetSystemMetrics(SM_CXSCREEN) / 2) - (wmc.Width / 2);

	if (wmc.Height != displayMode.dmPelsHeight)
		Y = (GetSystemMetrics(SM_CYSCREEN) / 2) - (wmc.Height / 2);

	RECT rc{ 0, 0, wmc.Width, wmc.Height };
	AdjustWindowRect(&rc, WS_TILEDWINDOW, false);

	MainWnd = CreateWindowEx(NULL, classstitle.c_str(), windowstitle.c_str(), WS_TILEDWINDOW, X, Y, rc.right - rc.left, rc.bottom - rc.top, nullptr, nullptr, hInst, nullptr);

	if(!MainWnd) {
		Logger("\tMain Window creation failed");

		return false;
	}
	
	ShowWindow(MainWnd, SW_NORMAL);
	UpdateWindow(MainWnd);

	Start();

	Log_Write("\tWindow Created");
	
	MainLoop();
		
	return true;
}

int WMainProg::MainLoop() {

	while (1) {
		if (PeekMessage(&uMsg, NULL, 0, 0, PM_REMOVE)) {
			TranslateMessage(&uMsg);
			DispatchMessage(&uMsg);

			if (uMsg.message == WM_QUIT) {
				break;
			}

			wndproc(uMsg);
		}

		Loop();
	}

	return static_cast<int> (uMsg.wParam);
}