#include <Graphics\Graphics3D.h>

#include <WinMain.h>

#include <Log\Logger.h>
#include <Config.h>

#include <Windows.h>

#include <iostream>
#include <string>
#include <memory>

const short St1{ 100 };
const short St2{ 101 };
const short Bt1{ 102 };

const short St4{ 103 };
const short St5{ 104 };

class Entry : public WMainProg {
public:
	std::unique_ptr<D12Graphics> Gfx;
public:
	Entry() { Log_Open("Logger.txt"); Log_Write("** Program Start **"); };
	~Entry() {}

	void wndproc(MSG uMsg);

	void Loop();
	void Start();
};

void Entry::wndproc(MSG uMsg) {
	Gfx->GfxLoop(uMsg);
}

void Entry::Start() {
	SetWindowName(L"DirectX12 - User Interface");
	Gfx->InitGraphics(MainWnd);
	Gfx->SetBgkColor(0.1f, 0.1f, 0.1f, 1.0f);

	CreateUXWindow(STATIC, L"HelloWorld", 0, 10, 110, 250, 50, St1, 0);
	CreateUXWindow(STATIC, L"Miss you much", 0, 270, 110, 250, 50, St2, 0);
	CreateUXWindow(BUTTON, L"Bye Bye for now", 0, 530, 110, 250, 50, Bt1, 0);

	CreateUXWindow(STATIC, L"Mouse X: ", 0, 20, 20, 250, 30, St4, 0);
	CreateUXWindow(STATIC, L"Mouse Y:", 0, 20, 60, 250, 30, St5, 0);

	SetUXBorder(St1, false);
	SetUXBorderColor(St2, D2D1::ColorF::Red);
}

void Entry::Loop() {
	std::wstring XCoord = L"Mouse X: " + std::to_wstring(Gfx->Mouse.x);
	SetUXName(St4, XCoord);

	std::wstring YCoord = L"Mouse Y: " + std::to_wstring(Gfx->Mouse.y);
	SetUXName(St5, YCoord);

	Gfx->Render(); 
}

int WINAPI wWinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPWSTR, int) {
	auto main = std::make_unique <Entry>();

	main->Gfx = std::make_unique<D12Graphics>();

	if (!main->Gfx->CheckSupport()) {
		std::cout << "No Support for DirectX12\n";
		Log_Write("\tNo Support for DirectX12\n");

		return 1;
	}
	
	main->Gfx->CheckVersionSupport();

	Config cfg{};

	LoadConfig(cfg, L"Config.cfg");
	if (!main->Gfx->GetResolutions(cfg)) {
		Log_Write("\tScreen Res failed\n");

		return 1;
	}

	SaveConfig(cfg, L"Config.cfg");

	WMainClass wmc{};

	wmc.Width = cfg.Width;
	wmc.Height = cfg.Height;

	main->Create(wmc);

	return 0;
}